/*
 * Project name:
     Hall current click (Using mikroE's Current click board)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Revision History:
     20140714:
       - initial release (FJ);
 * Description:
     This project is a simple demonstration of working with the Hall Current Click board
     based on the TLI4970 high precision miniature coreless magnetic current sensorwith SPI Interface.
     This example demosntrates measuring the current drawn by the consumer.
 * Test configuration:
     MCU:             STM32F107VC
                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/datasheet/CD00220364.pdf
     Dev. Board:      EasyMx PRO v7 for STM32
                      http://www.mikroe.com/easymx-pro/stm32/
     Oscillator:      HS-PLL 72.0000 MHz, internal 8.0000 MHz RC
     Ext. Modules:    Hall Current click board  - ac:Hall_click
                      http://www.mikroe.com/click/hall-current/
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/ARM/
 * NOTES:
     - Place Hall current click board in the mikroBUS socket 1.
     - Turn on TFT backlight on SW11.
     - Maximal SPI clock frequency for TLI4970 running at 3.3V is 5MHz.
         - To simulate analog input on ADC channel 3, use on-board potentiometer P1
       by connecting jumper J9 (board specific) to PA3(ADC channel 3 input).
 */
#define CLASSIC   1
#define GRAPHIQUE 0

#include "resources.h"

sbit Chip_Select at GPIOD_ODR.B13;

int j = 0;
char txt[64];

char txt_A_eff[15], txt_A_moy[15];
int current_read;
float current_read_float, current_abs_float, current_eff, current_moy;
float current_min, current_max;
unsigned char i[320];
unsigned char im[320];

char txt_W[15];
float power_float, Pmoy;

char txt_V[15];
float voltage_read_float;
//unsigned char v[320];

float k = 1.0; // chelle du graphique

// Get current value
unsigned Hall_Read()
{
  // Local variable
  unsigned int tmp = 0;
  unsigned int buffer = 0;

  Chip_Select = 0;          // Select Hall Click Board
  tmp = SPI3_Read(buffer);  // Get value
  Chip_Select = 1;          // Deselect Hall Click Board
  
  return tmp; // Returns value
}
// Get ADC value
float ADC_Read()
{
    int adc_;
    float adc_v;
    
    adc_ = ADC1_Get_Sample(3);            // Get ADC value from corresponding channel
    // x100 pour que 0  3.3V coresponde 0  330V
    // 0,08056640625 = 100.0 * 3.3/4096.0
    adc_v = (float)adc_ * 0.08056640625;
    return adc_v;
}

void Init()
{
  ADC_Set_Input_Channel(_ADC_CHANNEL_3);    // Choose ADC channel 3
  ADC1_Init();                              // Init ADC

 // Initialize 320x240 TFT display
  TFT_Set_Default_Mode();
  TFT_Init_ILI9341_8bit(320, 240);
  /*
  TFT_Set_Default_Mode();
  TFT_Init(320, 240);
*/
}

void DrawFrame()
{
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_BLACK, 1);
  TFT_Line(20,  46, 300,  46);
  TFT_Line(20, 220, 300, 220);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("Hall Click Board Demo", 55, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyMx PRO v7 for STM32", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  #if GRAPHIQUE
  Delay_ms(1000);
  TFT_Fill_Screen(CL_WHITE);
  #endif
}

void main()
{
 int x;
  Init();
  // Draw Graphical Frame
  DrawFrame();
  // Set chip select pin to be output
  GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_13);
  Chip_Select = 1;                 // Deselect Hall Click Board

  // Initialize SPI3 module
  SPI3_Init_Advanced(_SPI_FPCLK_DIV32, _SPI_MASTER | _SPI_16_BIT |
                     _SPI_CLK_IDLE_LOW | _SPI_SECOND_CLK_EDGE_TRANSITION | 
                     _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | 
                     _SPI_SSI_1, &_GPIO_MODULE_SPI3_PC10_11_12);
  Delay_ms(100);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL); // TFT_defaultFont
  // Display string on the TFT
  #if CLASSIC
  TFT_Write_Text("Voltage (V)  :", 55, 49); // 60
  TFT_Write_Text("Power  (VA) :", 55, 94); // 105
  #endif
  current_min = 0.0;
  current_max = 0.0;
  current_moy = 0.0;
  current_eff = 0.0;
  Pmoy = 0.0;
  k = 1.0;  // 50 pixels pour 50A
  // k = 50 si current_moy < 1A
  
  while(1)
  { 
   // Read voltage from ADC
    voltage_read_float = ADC_Read();
   // Read current from Hall Click
    current_read = Hall_Read();
    // If Sensor Current Value Transmission
    if(current_read.B15 == 0)
    {
      current_read = current_read & 0x1FFF;                     // Get [13:0] bits of current value
      current_read = current_read - 4096;                       // Map the current value
      current_read_float = (float)current_read * 0.0125;        // Calculate the current value in Amperes
      current_abs_float = fabs( current_read_float );
      current_eff = current_eff + pow(current_read_float, 2.0);
      //current_min = 0.0;
      if(current_abs_float > current_max) current_max = current_abs_float;
      // 1/320 = 0,003125
      current_moy = (319.0 * current_moy + current_abs_float) * 0.003125;
      power_float = voltage_read_float * current_moy;    // W = V * A
      // x va de 0  320 : 320 points
      x = j;
      TFT_Dot(x, i[j], CL_WHITE);
      TFT_Dot(x, im[j], CL_WHITE);
      i[j]  = 165 + k * current_read_float;
      im[j] = 215 - k * current_abs_float;
      TFT_Dot(x, 165, CL_GRAY);
      // Y graphique : 115  215 --> 100 pixel de hauteur centr sur 165
      TFT_Dot(x, i[j], CL_GREEN);//CL_OLIVE);
      // 100 pixel pour 50 ampres : 0.002
      TFT_Dot(x, im[j], CL_BLUE);
      j++;
    }

    if( j == 320 )
    {
      current_eff = sqrt(current_eff / 320.0);
      // Effacement du texte
      TFT_Set_Font(&Verdana12x13_Regular, CL_WHITE, FO_HORIZONTAL); // TFT_defaultFont
      #if CLASSIC
      TFT_Write_Text(txt_V, 200, 49); //60
      TFT_Write_Text(txt_A_eff, 200, 64); // 75
      TFT_Write_Text(txt_A_moy, 200, 79); // 90
      TFT_Write_Text(txt_W, 200, 94); //105
      #endif
      #if GRAPHIQUE
      TFT_Write_Text(txt, 19, 223);
      #endif
      // 
      TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL); // TFT_defaultFont
      //
      if(current_moy <= 1.0)
      {
       #if CLASSIC
       TFT_Write_Text("Current Eff (mA) :", 55, 64); // 75
       TFT_Write_Text("Current Moy (mA) :", 55, 79); // 90
       FloatToStr(1000.0 * current_eff, txt_A_eff);     // Convert the current value to a string
       FloatToStr(1000.0 * current_moy, txt_A_moy); // Convert the current value to a string
       #endif
       k = 50.0; // 50 = 1000.0 * 50pix / 1000mA;
      }
      else
      {
       #if CLASSIC
       TFT_Write_Text("Current Eff ( A) :", 55, 64); // 75
       TFT_Write_Text("Current Moy ( A) :", 55, 79); // 90
       FloatToStr(current_eff, txt_A_eff);     // Convert the current value to a string
       FloatToStr(current_moy, txt_A_moy); // Convert the current value to a string
       #endif
       k = 1.0; // 1 = 50 pix / 50A
      }
      //k = 50.0/(current_max - current_min);
      //k = 1.0;
      current_max = 0.0;
      #if CLASSIC
      FloatToStr(voltage_read_float, txt_V);          // Convert the current value to a string
      FloatToStr(power_float, txt_W);                  // Convert the power value to a string
      // Display the voltage value
      TFT_Write_Text(txt_V, 200, 49); //60
      // Display the current value
      TFT_Write_Text(txt_A_eff, 200, 64); // 75
      TFT_Write_Text(txt_A_moy, 200, 79); // 90
      // Display the power value
      TFT_Write_Text(txt_W, 200, 94); //105
      #endif
      #if GRAPHIQUE
      // Format values and store it to txt
      sprintf(txt, "%.3fV,  %.3fA Eff,  %.3fA Moy,  %.1fVA  ", voltage_read_float, current_eff, current_moy, power_float);
      TFT_Write_Text(txt, 19, 223);
      #endif
      j = 0;
      current_eff = 0.0;
    }
  }
}